# Copyright 2017-present Open Networking Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import, print_function

import json
import uuid
import random
import yaml
import glob
import string


class CORDDictUtils(object):
    @staticmethod
    def listToDict(alist, intListIndex):
        dictInfo = alist[int(intListIndex)]
        return dictInfo

    @staticmethod
    def jsonToList(strFile, strListName):
        data = json.loads(open(strFile).read())
        # print "data...",data
        dataList = data[strListName]
        return dataList

    def readFile(self, path, single=True):
        dataDict = {}
        for fileName in glob.glob(path):
            print("Reading ", fileName)
            data = open(fileName).read()
            dataDict[fileName] = data
            if bool(single):
                return data
        if not dataDict:
            print("Failed to find the file!")
            return None
        return dataDict

    def readFiles(self, path):
        return self.readFile(path, single=False)

    """
    @method compare_dict
    @Description: validates if contents of dict1 exists in dict2
    @params: dict1 = input_data entered through api
             dict2 = retrieved data from GET method
    returns True if contents of dict1 exists in dict2
    """

    def compare_dict(self, dict1, dict2):
        print("input data", dict1)
        print("get data", dict2)
        if dict1 is None or dict2 is None:
            return False
        if not isinstance(dict1, dict) or not isinstance(dict2, dict):
            return False
        if dict1 == {}:
            return True
        return self.compare_dict_recursive(dict1, dict2)

    """
    @method compare_dict_recursive
    @Description: recursive function to validate if dict1 is a subset of dict2
    returns True if contents of dict1 exists in dict2
    """

    def compare_dict_recursive(self, dict1, dict2):
        for key1, value1 in dict1.items():
            if key1 not in dict2.keys():
                print("Missing key", key1, "in dict2")
                return False
            value2 = dict2[key1]
            if isinstance(value1, dict) and isinstance(value2, dict):
                if not self.compare_dict_recursive(value1, value2):
                    return False
            else:
                if value2 != value1:
                    print("Values of key", key1, "in two dicts are not equal")
                    return False
        return True

    """
    @method compare_list_of_dicts
    @Description: validates if contents of dicts in list1 exists in dicts of list2
    returns True if for each dict in list1, there's a dict in list2 that contains its content
    """

    def compare_list_of_dicts(self, list1, list2):
        for dict1 in list1:
            if dict1 == {}:
                continue
            key = dict1.keys()[0]
            value = dict1[key]
            dict2 = self.getDictFromListOfDict(list2, key, value)
            if dict2 == {}:
                print(
                    "Comparison failed: no dictionaries found in list2 with key",
                    key,
                    "and value",
                    value,
                )
                return False
            if not self.compare_dict(dict1, dict2):
                print(
                    "Comparison failed: dictionary",
                    dict1,
                    "is not a subset of dictionary",
                    dict2,
                )
                return False
        return True

    """
    @method search_dictionary
    @Description: Searches for a key in the provided nested dictionary
    @params: input_dict = dictionary to be searched
             search_key = name of the key to be searched for
    returns two values: search_key value and status of the search.
             True if found (False when not found)

    """

    def search_dictionary(self, input_dict, search_key):
        input_keys = input_dict.keys()
        key_value = ""
        found = False
        for key in input_keys:
            if key == search_key:
                key_value = input_dict[key]
                found = True
                break
            elif isinstance(input_dict[key], dict):
                key_value, found = self.search_dictionary(
                    input_dict[key], search_key)
                if found:
                    break
            elif isinstance(input_dict[key], list):
                if not input_dict[key]:
                    found = False
                    break
                for item in input_dict[key]:
                    if isinstance(item, dict):
                        key_value, found = self.search_dictionary(
                            item, search_key)
                        if found:
                            break
        return key_value, found

    """
    @method getDictFromListOfDict
        return key_value,found
    @Description: Searches for the dictionary in the provided list of dictionaries
                  that matches the value of the key provided
    @params : List of dictionaries(getResponse Data from the URL),
             SearchKey - Key that needs to be searched for (ex: account_num)
             searchKeyValue - Value of the searchKey (ex: 21)
    @Returns: Dictionary returned when match found for searchKey with the corresponding
             searchKeyValue provided
    """

    def getDictFromListOfDict(self, getJsonDataList,
                              searchKey, searchKeyValue):
        return_dict = {}
        result = ""
        for data in getJsonDataList:
            print("data", data)
            return_dict = {}
            found = False
            input_keys = data.keys()
            for key in input_keys:
                if key == searchKey and str(data[key]) == str(searchKeyValue):
                    found = True
                    return_dict = data
                    print("return_dict", return_dict)
                    break
                elif isinstance(data[key], dict):
                    result, found = self.search_dictionary(
                        data[key], searchKey)
                    if found and str(result) == str(searchKeyValue):
                        return_dict = data
                        break
                elif isinstance(data[key], list):
                    for item in data[key]:
                        if isinstance(item, dict):
                            result, found = self.search_dictionary(
                                data[key], searchKey)
                            if found and str(
                                    result) == str(searchKeyValue):
                                return_dict = data
                                break
            if return_dict:
                break
        return return_dict

    """
    @method getFieldValueFromDict
    @params : search_dict - Dictionary to be searched
             field - Key to be searched for (ex: account_num)
    @Returns: Returns the value of the Key that was provided
    """

    def getFieldValueFromDict(self, search_dict, field):
        results = ""
        found = False
        input_keys = search_dict.keys()
        for key in input_keys:
            print("key...", key)
            if key == field:
                results = search_dict[key]
                if not results:
                    found = True
                    break
            elif isinstance(search_dict[key], dict):
                results, found = self.search_dictionary(
                    search_dict[key], field)
                if found:
                    break
            elif isinstance(search_dict[key], list):
                if not search_dict[key]:
                    found = False
                    continue
                for item in search_dict[key]:
                    if isinstance(item, dict):
                        results, found = self.search_dictionary(item, field)
                        if found:
                            break
            if results:
                break

        return results

    def setFieldValueInDict(self, input_dict, field, field_value):
        input_dict[field] = field_value
        return input_dict

    """
    @method getAllFieldValues
    @params : getJsonDataDictList - List of dictionaries to be searched
             fieldName - Key to be searched for (ex: instance_id)
    @Returns: Returns the unique value of the Key that was provided
    """

    def getAllFieldValues(self, getJsonDataDictList, fieldName):
        value_list = []
        # uniqValue = ""  - this is unused, commented out
        uniq_list = []
        for data in getJsonDataDictList:
            fieldValue = ""
            fieldValue = self.getFieldValueFromDict(data, fieldName)
            value_list.append(fieldValue)
        uniq_list = sorted(set(value_list))
        if len(uniq_list) == 1:
            pass  # see above, unused?
            # uniqValue = uniq_list[0]
        else:
            print("list of values found for ", fieldName, ";", uniq_list)
        return fieldValue

    def generate_uuid(self):
        return uuid.uuid4()

    def generate_random_number_from_blacklist(
        self, blacklist, min=100, max=500, typeTag=False
    ):
        num = None
        while num in blacklist or num is None:
            num = random.randrange(int(min), int(max))
        if typeTag:
            return num
        else:
            return str(num)

    def get_dynamic_resources(self, inputfile, resource):
        resourceNames = []
        names = {}
        dnames = []
        with open(inputfile, "r") as f:
            contents = yaml.load(f)
        resources = contents[resource]
        for i in resources:
            resourceNames.append(i["name"])
        for i in resourceNames:
            names["name"] = i
            dnames.append(names.copy())
        return dnames

    def generate_random_value(
            self, value, max_length=10, min_int=1, max_int=10000):
        if value == "string":
            return "".join(
                random.choice(string.ascii_lowercase + string.digits)
                for _ in range(max_length)
            )
        if value == "bool":
            return random.choice([True, False])
        if value == "int32" or value == "uint32":
            return random.randint(min_int, max_int)
        if value == "float":
            return random.uniform(1, 10)
        if value == "role":
            return "admin"
        if value == "direction":
            return random.choice(["in", "out"])
        if value == "flavor":
            return random.choice(["m1.large", "m1.medium", "m1.small"])
        if value == "vlan_tag":
            return random.choice(["555", "1-4096", "ANY"])
        if value == "ip_address":
            return ".".join(str(random.randint(0, 255)) for _ in range(4))
        else:
            return None

    def generate_random_slice_name(self):
        random_name = "".join(
            random.choice(
                string.ascii_lowercase +
                string.digits) for _ in range(10))
        return "testloginbase" + random_name



# Copyright 2017-present Open Networking Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import os

from .CORDDictUtils import CORDDictUtils
from .restApi import restApi
from .testCaseUtils import TestCaseUtils


# return the library version
def _version_():
    with open(os.path.join(os.path.dirname(__file__), "VERSION")) as f:
        return f.read().strip()


# Inherit all the other sub-classes
class CORDRobot(CORDDictUtils, restApi, TestCaseUtils):

    ROBOT_LIBRARY_SCOPE = "GLOBAL"

    def cr_version(self):
        return _version_()



# Copyright 2017-present Open Networking Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import, print_function

import requests
import json
import os

# These are the default values used with XOS
restApiDefaults = {
    'ATT_SERVICE': '/xosapi/v1/att-workflow-driver/attworkflowdriverservices',
    'ATT_SERVICEINSTANCES': '/xosapi/v1/att-workflow-driver/attworkflowdriverserviceinstances',
    'ATT_WHITELIST': '/xosapi/v1/att-workflow-driver/attworkflowdriverwhitelistentries',
    'BNG_MAP': '/xosapi/v1/fabric-crossconnect/bngportmappings',
    'CH_CORE_DEPLOYMENTS': '/xosapi/v1/core/deployments',
    'CH_CORE_FLAVORS': '/xosapi/v1/core/flavors',
    'CH_CORE_IMAGES': '/xosapi/v1/core/images',
    'CH_CORE_INSTANCES': '/xosapi/v1/core/instances',
    'CH_CORE_NETWORKS': '/xosapi/v1/core/networks',
    'CH_CORE_NETWORK_SLICES': '/xosapi/v1/core/networkslices',
    'CH_CORE_NETWORK_TEMPLATES': '/xosapi/v1/core/networktemplates',
    'CH_CORE_NODES': '/xosapi/v1/core/nodes',
    'CH_CORE_PORTS': '/xosapi/v1/core/ports',
    'CH_CORE_SERVICELINK': '/xosapi/v1/core/serviceinstancelinks',
    'CH_CORE_SERVICES': '/xosapi/v1/core/services',
    'CH_CORE_SERVICES': '/xosapi/v1/core/services',
    'CH_CORE_SITEDEPLOYMENTS': '/xosapi/v1/core/sitedeployments',
    'CH_CORE_SITES': '/xosapi/v1/core/sites',
    'CH_CORE_SLICES': '/xosapi/v1/core/slices',
    'CH_CORE_SLICES': '/xosapi/v1/core/slices',
    'CH_CORE_USERS': '/xosapi/v1/core/users',
    'CORE_DEPLOYMENTS': '/api/core/deployments/',
    'CORE_FLAVORS': '/api/core/flavors/',
    'CORE_IMAGES': '/api/core/images/',
    'CORE_INSTANCES': '/api/core/instances/',
    'CORE_NODES': '/api/core/nodes/',
    'CORE_SANITY_INSTANCES': '/api/core/instances/?no_hyperlinks=1',
    'CORE_SANITY_NODES': '/api/core/nodes/?no_hyperlinks=1',
    'CORE_SANITY_SLICES': '/api/core/slices/?no_hyperlinks=1',
    'CORE_SERVICES': '/api/core/services/',
    'CORE_SITEDEPLOYMENTS': '/api/core/sitedeployments',
    'CORE_SITES': '/api/core/sites/',
    'CORE_SLICES': '/api/core/slices/',
    'CORE_USERS': '/api/core/users/',
    'FABRIC_CROSSCONNECT_SERVICEINSTANCES': '/xosapi/v1/fabric-crossconnect/fabriccrossconnectserviceinstances',
    'FABRIC_SWITCH': '/xosapi/v1/fabric/switches',
    'HWVSG_TENANT': '/xosapi/v1/vsg-hw/vsghwserviceinstances',
    'ONU_DEVICE': '/xosapi/v1/volt/onudevices',
    'OSS_SERVICE': '/xosapi/v1/hippie-oss/hippieossservices',
    'OSS_SERVICEINSTANCE': '/xosapi/v1/hippie-oss/hippieossserviceinstances',
    'OSS_VOLT': '/xosapi/v1/core/servicedependencys',
    'PON_PORT': '/xosapi/v1/volt/ponports',
    'PORT_INTERFACE': '/xosapi/v1/fabric/portinterfaces',
    'SERVER_IP': '127.0.0.1',
    'SERVER_PORT': '30006',
    'SWITCH_PORT': '/xosapi/v1/fabric/switchports',
    'TENANT_SUBSCRIBER': '/api/tenant/cord/subscriber/',
    'TENANT_VOLT': '/api/tenant/cord/volt/',
    'UTILS_LOGIN': '/api/utility/login/',
    'UTILS_SYNCHRONIZER': '/api/utility/synchronizer/',
    'VOLT_DEVICE': '/xosapi/v1/volt/oltdevices',
    'VOLT_SERVICE': '/xosapi/v1/volt/voltservices',
    'VOLT_SUBSCRIBER': '/xosapi/v1/rcord/rcordsubscribers',
    'VOLT_TENANT': '/xosapi/v1/volt/voltserviceinstances',
    'VSG_TENANT': '/xosapi/v1/vsg/vsgserviceinstances',
    'XOS_PASSWD': 'letmein',
    'XOS_USER': 'admin@opencord.org',
}

jsonHeader = {"Content-Type": "application/json"}


class restApi():
    """
    Functions for testing CORD API with POST, GET, PUT, DELETE method
    """

    def getEnvOrDefault(self, key):
        """
        Find a variable in environment, or use Default value
        """
        if key in os.environ:
            value = os.environ[key]
        elif key in restApiDefaults:
            value = restApiDefaults[key]
        else:
            print("Unable to find '%s' in environment or defaults!" % key)
            value = None

        return value

    def getURL(self, key):
        """
        Get REST API suffix from key and return the full URL
        """
        urlSuffix = self.getEnvOrDefault(key)
        url = "http://" + self.getEnvOrDefault("SERVER_IP") + ":" + self.getEnvOrDefault("SERVER_PORT") + urlSuffix
        return url

    def checkResult(self, resp, expectedStatus):
        """
        Check if the status code in resp equals to the expected number.
        Return True or False based on the check result.
        """
        if resp.status_code == expectedStatus:
            print("Test passed: " + str(resp.status_code) + ": " + resp.text)
            return True
        else:
            print("Test failed: " + str(resp.status_code) + ": " + resp.text)
            return False

    def ApiPost(self, key, jsonData):
        url = self.getURL(key)
        data = json.dumps(jsonData)
        print("url, data..", url, data)
        resp = requests.post(
            url, data=data, headers=jsonHeader,
            auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD"))
        )
        print("requests.codes.....", requests.codes.created)
        passed = self.checkResult(resp, requests.codes.created) or self.checkResult(
            resp, requests.codes.ok
        )
        return passed

    def ApiPostReturnJson(self, key, jsonData):
        url = self.getURL(key)
        data = json.dumps(jsonData)
        print("url, data..", url, data)
        resp = requests.post(
            url, data=data, headers=jsonHeader,
            auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD"))
        )
        print("requests.codes.....", requests.codes.created)
        print("posted data...", resp.json())
        passed = self.checkResult(resp, requests.codes.created) or self.checkResult(
            resp, requests.codes.ok
        )
        return passed, resp.json()

    def ApiGet(self, key, urlSuffix=""):
        url = self.getURL(key) + str(urlSuffix)
        print("get url...", url)
        resp = requests.get(url, auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD")))
        passed = self.checkResult(resp, requests.codes.ok)
        if not passed:
            return None
        else:
            return resp.json()

    def ApiChameleonGet(self, key, urlSuffix=""):
        url = self.getURL(key) + "/" + str(urlSuffix)
        print("get url...", url)
        resp = requests.get(url, auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD")))
        passed = self.checkResult(resp, requests.codes.ok)
        if not passed:
            return None
        else:
            return resp.json()

    def ApiPut(self, key, jsonData, urlSuffix=""):
        print("urlSuffix....", type(urlSuffix))
        url = self.getURL(key) + str(urlSuffix) + "/"
        data = json.dumps(jsonData)
        resp = requests.put(
            url, data=data, headers=jsonHeader,
            auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD"))
        )
        passed = self.checkResult(resp, requests.codes.ok)
        return passed

    def ApiChameleonPut(self, key, jsonData, urlSuffix=""):
        print("urlSuffix....", type(urlSuffix))
        url = self.getURL(key) + "/" + str(urlSuffix)
        print("url", url)
        data = json.dumps(jsonData)
        resp = requests.put(
            url, data=data, headers=jsonHeader,
            auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD"))
        )
        passed = self.checkResult(resp, requests.codes.ok)
        return passed

    def ApiDelete(self, key, urlSuffix=""):
        url = self.getURL(key) + str(urlSuffix)
        print("url", url)
        resp = requests.delete(url, auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD")))
        passed = self.checkResult(resp, requests.codes.no_content)
        return passed

    def ApiChameleonDelete(self, key, urlSuffix=""):
        url = self.getURL(key) + "/" + str(urlSuffix)
        print("url", url)
        resp = requests.delete(url, auth=(self.getEnvOrDefault("XOS_USER"), self.getEnvOrDefault("XOS_PASSWD")))
        passed = self.checkResult(resp, requests.codes.created) or self.checkResult(
            resp, requests.codes.ok
        )
        return passed



#
# Copyright 2018 the original author or authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
Test Case Utils module
"""

from __future__ import absolute_import

import time
import subprocess
import pexpect
import sys


class TestCaseUtils():

    @staticmethod
    def config_dirs(self, log_dir, root_dir=None, voltha_dir=None):
        self.dirs["log"] = log_dir
        self.dirs["root"] = root_dir
        self.dirs["voltha"] = voltha_dir

    def get_dir(self, directory):
        return self.dirs.get(directory)

    @staticmethod
    def remove_leading_line(log_dir, log_file):
        with open(log_dir + "/" + log_file, "r+") as FILE:
            lines = FILE.readlines()
            FILE.seek(0)
            lines = lines[1:]
            for line in lines:
                FILE.write(line)
            FILE.truncate()
            FILE.close()

    @staticmethod
    def write_log_of_voltha_cli_comand(
        log_dir,
        log_file1,
        cmd1,
        log_file2=None,
        cmd2=None,
        log_file3=None,
        cmd3=None,
        host="localhost",
    ):
        output = open(log_dir + "/" + log_file1, "wb")
        child = pexpect.spawn(
            "ssh -p 30110 -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no voltha@%s"
            % host
        )
        child.expect(r"[pP]assword:")
        child.sendline("admin")
        child.expect(r"\((\x1b\[\d*;?\d+m){1,2}voltha(\x1b\[\d*;?\d+m){1,2}\)")
        time.sleep(10)
        child.sendline(cmd1)
        i = child.expect(
            [
                r"\((\x1b\[\d*;?\d+m){1,2}voltha(\x1b\[\d*;?\d+m){1,2}\)",
                r"\((\x1b\[\d*;?\d+m){1,2}.*device [0-9a-f]{16}(\x1b\[\d*;?\d+m){1,2}\)",
            ]
        )
        if i == 0:
            output.write(child.before)
            output.close()
            TestCaseUtils.remove_leading_line(log_dir, log_file1)
        elif i == 1:
            if log_file2 is not None and cmd2 is not None:
                output = open(log_dir + "/" + log_file2, "wb")
                child.sendline(cmd2)
                child.expect(
                    r"\((\x1b\[\d*;?\d+m){1,2}.*device [0-9a-f]{16}(\x1b\[\d*;?\d+m){1,2}\)"
                )
                output.write(child.before)
                output.close()
                TestCaseUtils.remove_leading_line(log_dir, log_file2)
            if log_file3 is not None and cmd3 is not None:
                output = open(log_dir + "/" + log_file3, "wb")
                child.sendline(cmd3)
                child.expect(
                    r"\((\x1b\[\d*;?\d+m){1,2}.*device [0-9a-f]{16}(\x1b\[\d*;?\d+m){1,2}\)"
                )
                output.write(child.before)
                output.close()
                TestCaseUtils.remove_leading_line(log_dir, log_file3)
        child.close()

    @staticmethod
    def write_log_of_onos_cli_command(log_dir, log_file, cmd, host="localhost", port=30115):
        output = open(log_dir + "/" + log_file, "wb")
        child = pexpect.spawn(
            "ssh -p %s -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no karaf@%s"
            % (port, host)
        )
        child.expect(r"[pP]assword:")
        child.sendline("karaf")
        # Expected prompt:
        #  onos>          (ONOS 1.x)
        #  karaf@root >   (ONOS 2.x)
        child.expect([r'(\x1b\[\d*;?\d+m){1,2}onos> (\x1b\[\d*;?\d+m){1,2}', r'karaf@root >'])
        child.sendline(cmd)
        child.expect([r'(\x1b\[\d*;?\d+m){1,2}onos> (\x1b\[\d*;?\d+m){1,2}', r'karaf@root >'])

        output.write(child.before)

        output.close()
        child.close()

    def get_fields_from_grep_command(self, search_word, log_file):
        grepCommand = "grep %s %s/%s" % (search_word, self.get_dir("log"), log_file)
        statusLines = subprocess.getstatusoutput(grepCommand)[1]
        return statusLines

    @staticmethod
    def parse_fields(status_line, delimiter):
        statusList = status_line.split(delimiter)
        return statusList

    def print_log_file(self, log_file):
        with open(self.get_dir("log") + "/" + log_file, "r+") as FILE:
            lines = FILE.readlines()
            print
            for line in lines:
                sys.stdout.write(line)

    @staticmethod
    def extract_pod_ip_addr(pod_name):
        proc1 = subprocess.Popen(
            ["/usr/bin/kubectl", "get", "svc", "--all-namespaces"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        proc2 = subprocess.Popen(
            ["grep", "-e", pod_name],
            stdin=proc1.stdout,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        proc3 = subprocess.Popen(
            ["awk", "{print $4}"],
            stdin=proc2.stdout,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        proc1.stdout.close()
        proc2.stdout.close()
        out, err = proc3.communicate()
        return out

    @staticmethod
    def extract_radius_ip_addr(pod_name):
        proc1 = subprocess.Popen(
            ["/usr/bin/kubectl", "describe", "pod", "-n", "voltha", pod_name],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        proc2 = subprocess.Popen(
            ["grep", "^IP:"],
            stdin=proc1.stdout,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        proc3 = subprocess.Popen(
            ["awk", "{print $2}"],
            stdin=proc2.stdout,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        proc1.stdout.close()
        proc2.stdout.close()
        out, err = proc3.communicate()
        return out

    @staticmethod
    def extract_pod_name(short_pod_name):
        proc1 = subprocess.Popen(
            ["/usr/bin/kubectl", "get", "pods", "--all-namespaces"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        proc2 = subprocess.Popen(
            ["grep", "-e", short_pod_name],
            stdin=proc1.stdout,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        proc3 = subprocess.Popen(
            ["awk", "{print $2}"],
            stdin=proc2.stdout,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        proc1.stdout.close()
        proc2.stdout.close()
        out, err = proc3.communicate()
        return out

    def modify_radius_ip_in_json_using_sed(self, new_ip_addr):
        sedCommand = (
            "sed -i '/radiusIp/c\\      \"radiusIp\":\"'%s'\",' %s/tests/atests/build/aaa_json"
            % (new_ip_addr, self.get_dir("voltha"))
        )
        status = subprocess.getstatusoutput(sedCommand)[0]
        return status

    @staticmethod
    def discover_rg_pod_name():
        return TestCaseUtils.extract_pod_name("rg0").strip()

    @staticmethod
    def retrieve_authorized_users_device_id_and_port_number(status_line):
        fields = TestCaseUtils.parse_fields(status_line, ",")
        deviceField = fields[2].strip()
        deviceStr, equal, deviceId = deviceField.partition("=")
        device_Id = deviceId
        portField = fields[4].strip()
        portNumStr, equal, portNum = portField.partition("=")
        portNumber = portNum
        return device_Id, portNumber

    def add_subscriber_access(self, device_id, port_number):
        TestCaseUtils.write_log_of_onos_cli_command(
            self.get_dir("log"),
            "voltha_add_subscriber_access.log",
            "volt-add-subscriber-access %s %s" % (device_id, port_number),
        )



#!/usr/bin/env python

# Copyright 2020-present Open Networking Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from setuptools import setup
from shutil import copyfile

LIBRARY_NAME = "CORDRobot"


def version():
    # Copy VERSION file of parent to module directory if not found
    version_path = os.path.join(LIBRARY_NAME, "VERSION")
    if not os.path.exists(version_path):
        copyfile("../VERSION", version_path)
    with open(version_path) as f:
        return f.read().strip()


def parse_requirements(filename):
    # parse a requirements.txt file, allowing for blank lines and comments
    requirements = []
    for line in open(filename):
        if line and not line.startswith("#"):
            requirements.append(line)
    return requirements


setup(
    name="cord-robot",
    version=version(),
    description="CORD Project Robot Libraries and common Resources",
    author="CORD Developers",
    include_package_data=True,
    packages=[LIBRARY_NAME],
    package_data={
        LIBRARY_NAME: ["rf-resources/*.resource", "VERSION"]
    },
    install_requires=parse_requirements("requirements.txt"),
)
